from .base_event import BaseEvent
from .event_dispatcher import EventDispatcher
from .event_handler import EventHandler
from .garlic import Garlic
from .protocol import Protocol
from .subscriber import Subscriber



from typing import Callable

from .base_event import BaseEvent
from .utils import get_typed_signature


class EventHandler:
    def __init__(self, channel_path: str = None, channel_delimiter: str = None) -> None:
        self._channel_path = channel_path
        self._channel_delimiter = channel_delimiter
        self._subscriptions: dict[str, list[Callable]] = {}

    def subscribe(self, subscriber: Callable):
        event_name = self.get_subscriber_event_name(subscriber)
        channel_name = self.get_channel_name(event_name=event_name)

        if channel_name not in self._subscriptions:
            self._subscriptions[channel_name] = []

        self._subscriptions[channel_name].append(subscriber)

    def send(self, event: BaseEvent):
        channel_name = self.get_channel_name(event_name=event.__class__.__name__)

        if channel_name in self._subscriptions:
            for subscriber in self._subscriptions[channel_name]:
                subscriber(event)

    def __call__(self, event: BaseEvent):
        self.send(event)

    @staticmethod
    def get_subscriber_event_name(subscriber: Callable) -> str:
        signature = get_typed_signature(subscriber)
        signature_params = signature.parameters

        assert "event" in signature_params, "Subscriber should have an event parameter"

        event_class = signature_params["event"].annotation
        return event_class.__name__

    def get_channel_name(self, event_name: str) -> str:
        channel_name = f"{self._channel_path}{self._channel_delimiter}{event_name}"
        return channel_name



from typing import Callable

from garlic import BaseEvent, EventDispatcher, EventHandler
from garlic.protocol import Protocol
from garlic.types import DecoratedCallable


class Garlic:
    def __init__(
        self,
        protocol: str = Protocol.MEMORY,
        channel_path: str = None,
        channel_delimiter: str = ".",
        event_handler: EventHandler = None,
        event_dispatcher: EventDispatcher = None,
    ) -> None:
        self._protocol = protocol
        self._channel_path = channel_path
        self._channel_delimiter = channel_delimiter
        self._event_handler = event_handler
        self._event_dispatcher = event_dispatcher
        self._bootstrap()

    def subscribe(self) -> Callable[[DecoratedCallable], DecoratedCallable]:
        def decorator(func: DecoratedCallable) -> DecoratedCallable:
            self._event_handler.subscribe(
                subscriber=func,
            )
            return func

        return decorator

    def publish(self, event: BaseEvent):
        self._event_dispatcher(event=event)

    def _bootstrap(self):
        self._event_handler = self._event_handler or EventHandler()
        self._event_dispatcher = self._event_dispatcher or EventDispatcher(
            event_handler=self._event_handler
        )



import inspect
from typing import Any, Callable, ForwardRef


def get_typed_annotation(annotation: Any) -> Any:
    if isinstance(annotation, str):
        annotation = ForwardRef(annotation)
    return annotation


def get_typed_signature(call: Callable[..., Any]) -> inspect.Signature:
    signature = inspect.signature(call)
    typed_params = [
        inspect.Parameter(
            name=param.name,
            kind=param.kind,
            default=param.default,
            annotation=get_typed_annotation(param.annotation),
        )
        for param in signature.parameters.values()
    ]
    typed_signature = inspect.Signature(typed_params)
    return typed_signature



from datetime import datetime
from uuid import UUID, uuid4

from pydantic import BaseModel


class BaseEvent(BaseModel):
    id: UUID = uuid4()
    created_at: datetime = datetime.now()
    version: str = "1"
    payload: dict = None



from uuid import UUID

from garlic import BaseEvent


class Subscriber:
    _consumed_events: dict[UUID, BaseEvent]

    def __init__(self):
        self._consumed_events = {}

    def __call__(self, event: BaseEvent):
        self.__check_duplicated_event(event)
        self._consumed_events[event.id] = event

    def __check_duplicated_event(self, event):
        if event.id in self._consumed_events:
            raise ValueError(f"Event {event.id} already consumed")
